<?php

/**
 * ACL Action plugin, hooks in to the acl check hook.
 */

if(!defined('DOKU_INC')) die();
if(!defined('DOKU_PLUGIN')) define( 'DOKU_PLUGIN', DOKU_INC.'lib/plugins/' );
require_once( DOKU_PLUGIN.'action.php' );
require_once( DOKU_INC.'inc/auth.php' );


/**
 *
 */
class action_plugin_op5acl extends DokuWiki_Action_Plugin {
    function __construct() {
    }


    function register( &$controller ) {
        /* This should just be an overlay to default ACL... therefore AFTER */
        $controller->register_hook( 'AUTH_ACL_CHECK', 'AFTER', $this, 'aclcheck' );
    }

    /**
     * Tests if page id is accessable by user
     *
     * $event->data must contain an array of three elements:
     *   id of page, as string
     *   username as string
     *   groups as array of strings
     *
     * this event handler overrides the defualt handler, if containing ACL data, and sets $event->return to auth level.
     *
     * @author Max Sikström <max.sikstrom@op5.com>
     *
     * @param Doku_Event $event Event object
     * @param null       $param Not used
     */
    function aclcheck( $event, $param=null ) {
        global $auth;

        list( $id,$user,$groups ) = $event->data;

        if( !( $auth instanceof auth_plugin_authop5 ) ) {
            msg( 'op5 ACL plugin works only with auth_plugin_authop5 authentication module. Fallback to default ACL' );
            return;
        }

        $perm = AUTH_NONE;


        /* Handle path as an array here instead... */
        if( $id != '' ) {
            $path = explode( ':', $id );
        } else {
            $path = array();
        }

        /* If user has wiki administrator permission, give admin access */
        if( $auth->op5auth->authorized_for( 'wiki_admin' ) ) {
            $perm = AUTH_ADMIN;
        }
        /* Make sneaky index work for limited users; allow at least read access to hosts namespace */
        else if (count($path) == 1) {
            if ($auth->op5auth->authorized_for('wiki'))
                $perm = AUTH_UPLOAD;
            else
                $perm = AUTH_READ;
        }
        /* If you're 'wiki', you can edit all objects you're authorized for. If you're not, you can only read
         * pages you're authorized for */
        else if (count($path) == 2 && $auth->op5auth->get_user()->authorized_for_object($path[0], $path[1], false)) {
            if ($auth->op5auth->authorized_for('wiki'))
                $perm = AUTH_DELETE;
            else
                $perm = AUTH_READ;
        }
        /* This is how services are accessed */
        else if (count($path) == 3 && $path[0] == 'host' && $auth->op5auth->get_user()->authorized_for_object('service', array($path[1], $path[2]), false)) {
            if ($auth->op5auth->authorized_for('wiki'))
                $perm = AUTH_DELETE;
            else
                $perm = AUTH_READ;
        }

        /* Give access if not already given... (Don't deny access if given by another plugin) */
        if( $event->result < $perm ) {
            $event->result = $perm;
        }

        return;
    }
}
